#!/usr/bin/perl
##########################################################################
#                                                                        #
# Perl agi Script for Authenticate caller via RADIUS                     #
# Switzernet(c)2011                                                      #
#                                                                        #
##########################################################################
# my modules
use strict;
use warnings;
use Config::IniFiles;
use POSIX;
use threads;
use threads::shared;
##########################################################################
# Setting
my $config_dir = '/etc/astrad-V2/config';
my $lock_file = '/var/run/ast-ngrep.pid';
my $runas_user = 'root';
##########################################################################
# Globals
my ($nas_ip,$LN,$thr);
my $t_event : shared;
##########################################################################
# Check if already running
if( -e $lock_file ) {
    open(PID,$lock_file);
    my $pid=<PID>;
    close PID;
    chomp $pid;
    if( !-e "/proc/$pid" ) {
        print STDERR "Lock file present, but no process with pid=$pid.\n";
        die "Can't delete lock file $lock_file\n" if !unlink $lock_file;
        print STDERR "Lock file has been removed.\n";
    } else {
        die "Lockfile present, another copy is punning pid=$pid\n";
    }
}
my ($name, $passwd, $uid, $gid) = getpwnam($runas_user) or die "$runas_user not in passwd file";;
##########################################################################
# set C locale
POSIX::setlocale(LC_ALL, "C");
##########################################################################
# Become daemon
my $pid;
if( !defined($pid = fork()) ) 
    {
    die "cannot fork: $!";
    } 
elsif ($pid) 
    {
    # Create lockfile, and finish parent process
    open(PID, "> $lock_file") || die "ast-ngrep.pl: Unable to create lockfile $lock_file\n";
    print PID "$pid";
    close PID;
    chown $uid, $gid, $lock_file;
    exit;
    } 
    else 
        {
        # daemon
        setpgrp();
        select(STDERR); $| = 1;
        select(STDOUT); $| = 1;
        }
##########################################################################
#                                FUNCTION                                #
##########################################################################
# Load config
sub load_config {
    my $conf=Config::IniFiles->new(-file => "$config_dir/switzer.conf");
    return 0 if !defined $conf ;
    $nas_ip = $conf->val('GLOBAL','NAS_IP');    
    return 1;
}
##########################################################################
# thread control
sub NGREP_control {
my $l_event = 0;
while ( 1 ) {
    $l_event = time();
    if ($l_event - $t_event > 400) {
        unlink $lock_file or print ("Unable to create lockfile $lock_file\n");
        exit;
        }
    sleep 5;
    }
}
##########################################################################
#                                 main()                                 #
##########################################################################    
exit if !load_config();
print "SIP log Start\n";
$t_event = time();
$thr = threads->new(\&NGREP_control);
open COMMAND, '/usr/bin/ngrep -pql -W byline "" port 5060 |';
open LOG, ">/var/sipenv-$nas_ip/log/sip.log" || die ("Erreur d'ouverture du fichier /var/sipenv-*/log/sip.log") ;
while (<COMMAND>)
    {
    $LN=$_;
    chomp($LN);
    print LOG "\n$LN";
    $t_event = time();
    }
##########################################################################
#                                  END                                   #
##########################################################################
